@@ -36,7 +36,7 @@ impl GLM {
     }
 
     // Run Kalman Filter on instance variables
-    pub fn kalman_filter(&self) -> Result<(Array3<f64>, Array3<f64>, Array3<f64>, Array3<f64>, Array3<f64>), Box<dyn Error>> {
+    pub fn kalman_filter(&self) -> Result<(Array3<f64>, Array3<f64>, Array3<f64>, Array3<f64>, Array3<f64>, Array3<f64>, Array3<f64>), Box<dyn Error>> {
 
         let T = self.y.len();
         let p = self.T.ncols();
@@ -44,88 +44,114 @@ impl GLM {
 
         let mut axes_iterator: Array3<f64> = Array3::zeros((1, 1, T));
 
-        let mut a_3d: Array3<f64> = Array3::zeros((p, 1, T));
-        let mut v_3d: Array3<f64> = Array3::zeros((s, 1, T));
-        let mut F_3d: Array3<f64> = Array3::zeros((s, s, T));
-        let mut P_3d: Array3<f64> = Array3::zeros((p, p, T));
-        let mut K_3d: Array3<f64> = Array3::zeros((p, s, T));
+        let mut a_3d:   Array3<f64> = Array3::zeros((p, 1, T));
+        let mut att_3d: Array3<f64> = Array3::zeros((p, 1, T));
+        let mut v_3d:   Array3<f64> = Array3::zeros((s, 1, T));
+        let mut F_3d:   Array3<f64> = Array3::zeros((s, s, T));
+        let mut P_3d:   Array3<f64> = Array3::zeros((p, p, T));
+        let mut Ptt_3d: Array3<f64> = Array3::zeros((p, p, T));
+        let mut M_3d:   Array3<f64> = Array3::zeros((p, s, T));
+        let mut K_3d:   Array3<f64> = Array3::zeros((p, s, T));
         
-        let mut a_prev: Array2<f64> = Array2::zeros((p, 1));
-        let mut v_prev: Array2<f64> = Array2::zeros((s, 1));
-        let mut F_prev: Array2<f64> = Array2::zeros((s, s));
-        let mut P_prev: Array2<f64> = Array2::zeros((p, p));
-        let mut K_prev: Array2<f64> = Array2::zeros((p, s));
+        let mut a_prev:   Array2<f64> = Array2::zeros((p, 1));
+        let mut att_prev: Array2<f64> = Array2::zeros((p, 1));
+        let mut v_prev:   Array2<f64> = Array2::zeros((s, 1));
+        let mut F_prev:   Array2<f64> = Array2::zeros((s, s));
+        let mut P_prev:   Array2<f64> = Array2::zeros((p, p));
+        let mut Ptt_prev: Array2<f64> = Array2::zeros((p, p));
+        let mut M_prev:   Array2<f64> = Array2::zeros((p, s));
+        let mut K_prev:   Array2<f64> = Array2::zeros((p, s));
 
         // need to enumerate to use i
         for (i, _) in axes_iterator.axis_iter_mut(Axis(2)).enumerate() {
             
             // retrieve slices of the data
-            let mut a_temp: ArrayViewMut2<f64> = a_3d.slice_mut(s![..,..,i]);
-            let mut v_temp: ArrayViewMut2<f64> = v_3d.slice_mut(s![..,..,i]);
-            let mut F_temp: ArrayViewMut2<f64> = F_3d.slice_mut(s![..,..,i]);
-            let mut P_temp: ArrayViewMut2<f64> = P_3d.slice_mut(s![..,..,i]);
-            let mut K_temp: ArrayViewMut2<f64> = K_3d.slice_mut(s![..,..,i]);
+            let mut a_temp:   ArrayViewMut2<f64> = a_3d.slice_mut(s![..,..,i]);
+            let mut att_temp: ArrayViewMut2<f64> = att_3d.slice_mut(s![..,..,i]);
+            let mut v_temp:   ArrayViewMut2<f64> = v_3d.slice_mut(s![..,..,i]);
+            let mut F_temp:   ArrayViewMut2<f64> = F_3d.slice_mut(s![..,..,i]);
+            let mut P_temp:   ArrayViewMut2<f64> = P_3d.slice_mut(s![..,..,i]);
+            let mut Ptt_temp: ArrayViewMut2<f64> = Ptt_3d.slice_mut(s![..,..,i]);
+            let mut M_temp:   ArrayViewMut2<f64> = M_3d.slice_mut(s![..,..,i]);
+            let mut K_temp:   ArrayViewMut2<f64> = K_3d.slice_mut(s![..,..,i]);
             
             // in first iteration: set first values of a and P and compute corresponding v and F
             // TODO: add diffuse initialization
+            // TODO: add incasting
             if i == 0 {
 
-                // get y_0
-                let y_temp: ArrayView2<f64> = self.y.slice(s![.., .., i]);
-
                 // set a_0 and P_0
                 a_temp.assign(&arr2(&[[0.0], [0.0]]));
                 P_temp.assign(&arr2(&[[1.0, 0.0], [0.0, 1.0]]));
 
+                // get y_0
+                let y_temp: ArrayView2<f64> = self.y.slice(s![.., .., i]);
+
                 // get first error and error variance: v and F
                 v_temp.assign(&(
                     &y_temp - &self.Z.dot(&a_temp))
                 );
+                
                 F_temp.assign(&(
                     &self.Z.dot(&P_temp.dot(&self.Z.t())) + &self.H
                 ));
 
-                // compute Kalman gain
-                K_temp.assign(&(
-                    &self.T.dot(
-                        &P_temp.dot(
-                            &self.Z.t().dot(
-                                &F_temp.inv().unwrap()
-                            )
+                // compute incasted Kalman gain M
+                M_temp.assign(&(
+                    &P_temp.dot(
+                        &self.Z.t().dot(
+                            &F_temp.inv().unwrap()
                         )
                     )
                 ));
 
+                // compute Kalman gain K
+                K_temp.assign(&(
+                    &self.T.dot(
+                        &M_temp)
+                ));
+                
+                // compute incasted att
+                att_temp.assign(&(
+                    &a_temp + &M_temp.dot(&v_temp)
+                ));
+
+                // compute incasted Ptt
+                Ptt_temp.assign(&(
+                    &P_temp - 
+                        &M_temp.dot(&F_temp.dot(&M_temp.t()))
+                ));
+
                 // persist lagged a in memory
                 a_prev.assign(&a_temp);
                 P_prev.assign(&P_temp);
+                M_prev.assign(&M_temp);
                 K_prev.assign(&K_temp);
                 v_prev.assign(&v_temp);
                 F_prev.assign(&F_temp);
+                att_prev.assign(&att_temp);
+                Ptt_prev.assign(&Ptt_temp);
             }
 
             else {
 
                 // get new a_i and assign to mutable a_3d slice
                 a_temp.assign(&(
-                    &self.T.dot(&a_prev) + &K_prev.dot(&v_prev)
+                    &self.T.dot(&att_prev)
                 ));
 
                 // get new P_i and assign to mutable slide of P_3d
                 P_temp.assign(&
                     (&self.T.dot(
-                        &P_prev.dot(
+                        &Ptt_prev.dot(
                             &self.T.t()
                         )
                     ) + &self.R.dot(
                             &self.Q.dot(
                                 &self.R.t()
-                        )
-                    ) + &K_prev.dot(
-                            &F_prev.dot(
-                                &K_prev.t()
-                        )
-                    ))
+                            )
+                        ) 
+                    )
                 ); 
                 
                 // get current y
@@ -141,27 +167,45 @@ impl GLM {
                     &self.Z.dot(&P_temp.dot(&self.Z.t())) + &self.H
                 ));
 
-                // compute Kalman gain
-                K_temp.assign(&(
-                    &self.T.dot(
-                        &P_temp.dot(
-                            &self.Z.t().dot(
-                                &F_temp.inv().unwrap()
-                            )
+                // compute incasted Kalman gain M
+                M_temp.assign(&(
+                    &P_temp.dot(
+                        &self.Z.t().dot(
+                            &F_temp.inv().unwrap()
                         )
                     )
                 ));
 
+                // compute Kalman gain K
+                K_temp.assign(&(
+                    &self.T.dot(
+                        &M_temp)
+                ));
+
+                // compute incasted att
+                att_temp.assign(&(
+                    &a_temp + &M_temp.dot(&v_temp)
+                ));
+
+                // compute incasted Ptt
+                Ptt_temp.assign(&(
+                    &P_temp - 
+                        &M_temp.dot(&F_temp.dot(&M_temp.t()))
+                ));
+
                 // persist lagged a in memory
                 a_prev.assign(&a_temp);
                 P_prev.assign(&P_temp);
+                M_prev.assign(&M_temp);
                 K_prev.assign(&K_temp);
                 v_prev.assign(&v_temp);
                 F_prev.assign(&F_temp);
+                att_prev.assign(&att_temp);
+                Ptt_prev.assign(&Ptt_temp);
             }
         }
 
-        Ok((a_3d, P_3d, v_3d, F_3d, K_3d))
+        Ok((a_3d, att_3d, P_3d, Ptt_3d, v_3d, F_3d, K_3d))
 
     }
 
